#!/usr/bin/env python
import multiprocessing
import os
import subprocess

import rospy
import actionlib
from std_msgs.msg import String
import pdb

from alarm_watchdog_mapping_ros_node.msg import WDStartMappingAction,\
    WDEndMappingAction, \
    WDStartMappingActionGoal, \
    WDEndMappingActionGoal, \
    WDStartMappingActionResult, \
    WDEndMappingActionResult


def alarm_watchdog_mapping_ros_node_start_cb(goal: WDStartMappingActionGoal):
    global alarm_watchdog_mapping_ros_node_start

    # TODO initiate mapping process

    result = WDStartMappingActionResult()
    result.result.return_code = String("OK")
    rospy.loginfo('Mapping init request received')
    alarm_watchdog_mapping_ros_node_start.set_succeeded(result)

def alarm_watchdog_mapping_ros_node_end_cb(goal: WDEndMappingActionGoal):
    global alarm_watchdog_mapping_ros_node_end

    is_success = True
    # TODO finish mapping process
    # TODO if map processing failed set is_success to False

    result = WDEndMappingActionResult()
    result.result.return_code = String("OK" if is_success else "ERROR")
    rospy.loginfo('Mapping end request received')
    alarm_watchdog_mapping_ros_node_end.set_succeeded(result)

    if is_success:
        # TODO send in ROS topics point-cloud and the path
        # TODO in our case we just replay ROS bag file with these topics
        # we need /rtabmap/mapGraph and /master_cloud

        # play ros bag file with point-cloud and path (remove this when replaced with real data)
        try:
            rospy.loginfo('Sending point-cloud and path')
            PATH_ROS_BAG_FILE = "/home/ubuntu/Documents/maps/map_info_WH_with_tf_based_odom.bag"
            os.system("rosbag play " + PATH_ROS_BAG_FILE)
        except subprocess.CalledProcessError as e:
            rospy.logerror(f"Command failed with return code {e.returncode}")


def main():
    global alarm_watchdog_mapping_ros_node_start, alarm_watchdog_mapping_ros_node_end

    alarm_watchdog_mapping_ros_node_start = actionlib.SimpleActionServer(
        'alarm_watchdog_mapping_ros_node_start',
        WDStartMappingAction,
        execute_cb=alarm_watchdog_mapping_ros_node_start_cb,
        auto_start=False
    )
    alarm_watchdog_mapping_ros_node_start.start()
    rospy.loginfo("alarm_watchdog_mapping_ros_node_start server started")

    alarm_watchdog_mapping_ros_node_end = actionlib.SimpleActionServer(
        'alarm_watchdog_mapping_ros_node_end',
        WDEndMappingAction,
        execute_cb=alarm_watchdog_mapping_ros_node_end_cb,
        auto_start=False
    )
    alarm_watchdog_mapping_ros_node_end.start()
    rospy.loginfo("alarm_watchdog_mapping_ros_node_end server started")


if __name__ == '__main__':
    rospy.init_node('alarm_watchdog_mapping_ros_node')
    main()
